@@ -18,10 +18,16 @@ package com.clustercontrol.agent;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.net.Authenticator;
 import java.net.DatagramPacket;
 import java.net.DatagramSocket;
 import java.net.InetAddress;
 import java.net.NetworkInterface;
+import java.net.PasswordAuthentication;
+import java.net.ServerSocket;
+import java.net.Socket;
 import java.net.SocketException;
 import java.net.URL;
 import java.net.URLConnection;
@@ -39,11 +45,16 @@ import org.apache.log4j.helpers.LogLog;
 
 import com.clustercontrol.agent.log.LogfileMonitorManager;
 import com.clustercontrol.agent.util.AgentProperties;
+import com.clustercontrol.agent.util.PropertiesFileUtil;
+import com.clustercontrol.bean.PriorityConstant;
+import com.clustercontrol.fault.HinemosUnknown;
 import com.clustercontrol.repository.bean.AgentCommandConstant;
+import com.clustercontrol.util.Messages;
 import com.clustercontrol.ws.agent.AgentInfo;
 import com.clustercontrol.ws.agent.HinemosUnknown_Exception;
 import com.clustercontrol.ws.agent.InvalidRole_Exception;
 import com.clustercontrol.ws.agent.InvalidUserPass_Exception;
+import com.clustercontrol.ws.agent.OutputBasicInfo;
 
 /**
  * エージェントメインクラス<BR>
@@ -71,6 +82,14 @@ public class Agent {
 
 	public static final Integer DEFAULT_CONNECT_TIMEOUT = 10;
 	public static final Integer DEFAULT_REQUEST_TIMEOUT = 60;
+	
+	public static final String DEFAULT_PROXY_HOST = "";
+	public static final int DEFAULT_PROXY_PORT = 8080;
+	public static final String DEFAULT_PROXY_USER = "";
+	public static final String DEFAULT_PROXY_PASSWORD = "";
+	
+	private static final String REPLACE_VALUE_MANAGER_IP = "${ManagerIP}";
+	
 
 	/*
 	 * AgentHome
@@ -181,6 +200,67 @@ public class Agent {
 			}
 		}
 	}
+	
+	/**
+	 * マネージャのsendManagerIpを待つメソッド。
+	 * TCP 24005にパケットが送られてきたら、パケット内のメッセージ（IPアドレス）を返す。
+	 * @throws Exception 
+	 */
+	private String receiveManagerIp() throws Exception{
+		final int port = 24005;
+		ServerSocket servSock = null;
+		Socket clntSock = null;
+		
+		final int BUFSIZE = 256;
+		int tmpRecvMsgSize = 0;
+		int recvMsgSize = 0;
+		byte[] receiveBuf = new byte[BUFSIZE];
+		String recvMsg = "";
+		
+		try {
+			servSock = new ServerSocket(port);
+			
+			// マネージャからの接続を待つ
+			clntSock = servSock.accept();
+			m_log.info("connecting to： " + clntSock.getRemoteSocketAddress().toString());
+
+			InputStream in = clntSock.getInputStream();
+			OutputStream out = clntSock.getOutputStream();
+
+			while ((tmpRecvMsgSize = in.read(receiveBuf)) != -1) {
+				out.write(receiveBuf, 0, tmpRecvMsgSize);
+				recvMsgSize = tmpRecvMsgSize;
+			}
+			
+			recvMsg = new String(receiveBuf, 0, recvMsgSize);
+			m_log.info("receiveMessage : " + recvMsg);
+			
+			if(!(recvMsg.contains("agentFacilityId=") && recvMsg.contains("managerIp="))){
+				throw new HinemosUnknown("receive message is invalid");
+			}
+			
+			m_log.info("receive message: " + recvMsg);
+		} catch (Exception e) {
+			m_log.warn("receiveManagerIp " + e.getClass().getSimpleName() + ", " + e.getMessage());
+			throw e;
+		} finally {
+			try{
+				if(clntSock != null){
+					clntSock.close();
+				}
+			} catch(Exception e) {
+				m_log.warn("receiveManagerIp: " + e);
+			}
+			try{
+				if(servSock != null){
+					servSock.close();
+				}
+			} catch(Exception e) {
+				m_log.warn("receiveManagerIp: " + e);
+			}
+		}
+		return recvMsg;
+	}
 
 	/**
 	 * コンストラクタ
@@ -208,6 +288,13 @@ public class Agent {
 
 		int connectTimeout = DEFAULT_CONNECT_TIMEOUT;
 		int requestTimeout = DEFAULT_REQUEST_TIMEOUT;
+		
+		// プロキシ設定
+		String proxyHost = DEFAULT_PROXY_HOST;
+		int proxyPort = DEFAULT_PROXY_PORT;
+		String proxyUser = DEFAULT_PROXY_USER;
+		String proxyPassword = DEFAULT_PROXY_PASSWORD;
+		
 		try {
 			String strConnect = AgentProperties.getProperty("connect.timeout");
 			if (strConnect != null) {
@@ -217,10 +304,79 @@ public class Agent {
 			if (strRequest != null) {
 				requestTimeout = Integer.parseInt(strRequest);
 			}
+			String strProxyHost = AgentProperties.getProperty("http.proxy.host");
+			if (strProxyHost != null) {
+				proxyHost = strProxyHost;
+			}
+			String strProxyPort = AgentProperties.getProperty("http.proxy.port");
+			if (strProxyPort != null) {
+				proxyPort = Integer.parseInt(strProxyPort);
+			}
+			String strProxyUser = AgentProperties.getProperty("http.proxy.user");
+			if (strProxyUser != null) {
+				proxyUser = strProxyUser;
+			}
+			String strProxyPassword = AgentProperties.getProperty("http.proxy.password");
+			if (strProxyPassword != null) {
+				proxyPassword = strProxyPassword;
+			}
 		} catch (Exception e) {
-			m_log.warn("timeout : " + e.getMessage());
+			m_log.warn(e.getMessage());
 		}
+		
+		if(!"".equals(proxyHost)){
+			System.setProperty("http.proxyHost", proxyHost);
+			System.setProperty("http.proxyPort", Integer.toString(proxyPort));
+			BasicAuth basicAuth = new BasicAuth(proxyUser, proxyPassword);
+			Authenticator.setDefault(basicAuth);
+			m_log.info("proxy.host=" + System.getProperty("http.proxyHost") + ", proxy.port=" + System.getProperty("http.proxyPort") + ", proxy.user=" + proxyUser);
+		}
+		
+		// マネージャからの接続を待ち、接続先マネージャを指定する
+		String managerAddress = AgentProperties.getProperty("managerAddress");
+		URL url = new URL(managerAddress);
+		boolean replacePropFileSuccess = true;
+		String errMsg = "";
+		if(REPLACE_VALUE_MANAGER_IP.equals((url.getHost()))){
+			try{
+				m_log.info("waiting for manager connection...");
+				String recvMsg = receiveManagerIp();
+				
+				String managerIp = recvMsg.split(",")[0].split("=")[1];
+				String agentFacilityId = recvMsg.split(",")[1].split("=")[1];
 
+				String key = "managerAddress";
+				String value = url.getProtocol() + "://" + managerIp + ":" + url.getPort() + "/HinemosWS/";
+				m_log.info("key : " + key);
+				m_log.info("value : " + value);
+				// Agent.propertiesファイルの接続先マネージャ書き換え
+				PropertiesFileUtil.replacePropertyFile(propFileName, key, managerAddress, value);
+				AgentProperties.setProperty(key, value);
+				
+				key = "facilityId";
+				m_log.info("key : " + key);
+				m_log.info("value : " + agentFacilityId);
+				// Agent.propertiesファイルのエージェントファシリティID書き換え
+				PropertiesFileUtil.replacePropertyFile(propFileName, key, "", agentFacilityId);
+				AgentProperties.setProperty(key, agentFacilityId);
+				
+				key = "log4j.appender.syslog.SyslogHost";
+				// log4j.propertiesファイルの書き換え（Windows版エージェントのみ）
+				if(REPLACE_VALUE_MANAGER_IP.equals(AgentProperties.getProperty(key))){
+					m_log.info("key : " + key);
+					m_log.info("value : " + managerIp);
+					PropertiesFileUtil.replacePropertyFile(log4jFileName, key, REPLACE_VALUE_MANAGER_IP, managerIp);
+				}
+			} catch (HinemosUnknown e) {
+				// プロパティファイル書き換えに失敗した場合、マネージャに通知する
+				errMsg = e.getMessage();
+				m_log.warn(errMsg, e);
+				replacePropFileSuccess = false;
+			} catch (Exception e) {
+				m_log.warn(e.getMessage(), e);
+				throw e;
+			}
+		}
 
 		try {
 			EndpointManager.init(AgentProperties.getProperty("user"),
@@ -228,10 +384,26 @@ public class Agent {
 					AgentProperties.getProperty("managerAddress"),
 					connectTimeout, requestTimeout);
 		} catch (Exception e) {
-			m_log.error("EndPointWrapper.init error : " + e.getMessage(), e);
+			m_log.error("EndpointManager.init error : " + e.getMessage(), e);
 			m_log.error("current-dir=" + (new File(".")).getAbsoluteFile().getParent());
 			throw e;
 		}
+		
+		if(!replacePropFileSuccess){
+			OutputBasicInfo output = new OutputBasicInfo();
+			output.setPluginId("AGT_UPDATE_CONFFILE");
+			output.setPriority(PriorityConstant.TYPE_WARNING);
+			output.setApplication(Messages.getString("agent"));
+			output.setMessageId("001");
+			String[] args = {errMsg};
+			output.setMessage(Messages.getString("message.agent.replace.file.faulure.notify.msg"));
+			output.setMessageOrg(Messages.getString("message.agent.replace.file.faulure.notify.origmsg", args));
+			output.setGenerationDate(new Date().getTime());
+			output.setMonitorId("SYS");
+			output.setFacilityId(""); // マネージャがセットする。
+			output.setScopeText(""); // マネージャがセットする。
+			m_sendQueue.put(output);
+		}
 
 		Runtime.getRuntime().addShutdownHook(new Thread() {
 			@Override
@@ -527,3 +699,17 @@ public class Agent {
 		return agentHome;
 	}
 }
+
+class BasicAuth extends Authenticator{
+	 private String username;
+	 private String password;
+	 public BasicAuth(String username, String password)
+	 {
+		 this.username = username;
+		 this.password = password;
+	 }
+
+	 protected PasswordAuthentication getPasswordAuthentication() {
+		 return new PasswordAuthentication(username,password.toCharArray());
+	 }
+}
\ No newline at end of file
