@@ -29,6 +29,7 @@ import com.clustercontrol.ws.agent.MonitorNotFound_Exception;
 import com.clustercontrol.ws.agent.OutputBasicInfo;
 import com.clustercontrol.ws.agent.SettingUpdateInfo;
 import com.clustercontrol.ws.agent.TopicInfo;
+import com.clustercontrol.ws.common.cloud.CloudCommonEndpoint;
 import com.clustercontrol.ws.jobmanagement.RunResultInfo;
 import com.clustercontrol.ws.monitor.CommandExecuteDTO;
 import com.clustercontrol.ws.monitor.CommandResultDTO;
@@ -250,6 +251,22 @@ public class AgentEndPointWrapper {
 		}
 		throw wse;
 	}
+	
+	public static DataHandler downloadScripts(String filename)
+	throws com.clustercontrol.ws.common.cloud.HinemosUnknown_Exception, com.clustercontrol.ws.common.cloud.InvalidRole_Exception, com.clustercontrol.ws.common.cloud.InvalidUserPass_Exception {
+		WebServiceException wse = null;
+		for (EndpointSetting endpointSetting : EndpointManager.getCloudCommonEndpoint()) {
+			try {
+				CloudCommonEndpoint endpoint = (CloudCommonEndpoint) endpointSetting.getEndpoint();
+				return endpoint.downloadScripts(filename);
+			} catch (WebServiceException e) {
+				wse = e;
+				m_log.info("WebServiceException " + e.getMessage());
+				EndpointManager.changeEndpoint();
+			}
+		}
+		throw wse;
+	}
 
 	public static HashMap<String, String> getAgentLibMap()
 	throws HinemosUnknown_Exception, InvalidRole_Exception, InvalidUserPass_Exception  {
