@@ -69,6 +69,8 @@ public class LogfileMonitor {
 
 	private static final String PROGRAM = "log.msg.program";
 
+	private static final String FIRST_HEAD_OPEN = "log.first.head.open";
+
 	private static final String MESSAGE_ID_INFO = "001";
 
 	private static final String MESSAGE_ID_WARNING = "002";
@@ -93,9 +95,15 @@ public class LogfileMonitor {
 	/** 動作間隔（ミリ秒） */
 	private int m_runInterval;
 
-	/** 最初にファイルをチェック */
-	private boolean m_existenceFlg = true;
+	/** 初回ファイルチェック時にファイルが存在せず、初めてファイルを読み込んだ際に先頭から読むか否か */
+	private boolean m_firstHeadOpen = false;
 
+	/** ファイル先頭から読むか否かのステータス情報を保持するフラグ */
+	private boolean m_firstHeadOpenStatus = false;
+	
+	/** ログローテートを判定するフラグ */
+	private boolean m_logrotate = false;
+	
 	/** ファイル変更チェック期間設定（ミリ秒） */
 	private int m_unchangedStatsPeriod = 0;
 
@@ -148,7 +156,6 @@ public class LogfileMonitor {
 			String path, int runInterval) {
 		m_logfileMonitorManager = logfileManager;
 		m_filePath = path;
-		m_existenceFlg = true;
 		m_runInterval = runInterval;
 
 		// ファイル変更チェック期間（秒）
@@ -198,6 +205,14 @@ public class LogfileMonitor {
 		}
 		m_log.debug(PROGRAM + " = " + m_program);
 
+		// 初回読み込み時にログファイルが存在せず、初めにファイルを読み込んだ際に先頭からファイルを読むか否か
+		String firstHeadOpenString = AgentProperties.getProperty(FIRST_HEAD_OPEN, "false");
+		if("true".equals(firstHeadOpenString)){
+			m_firstHeadOpen = true;
+		}
+		m_firstHeadOpenStatus = m_firstHeadOpen;
+		m_log.debug(FIRST_HEAD_OPEN + " = " + m_firstHeadOpen);
+		
 		// ログファイルのエンコーディングを設定
 		m_logEncoding =  AgentProperties.getProperty(LOG_ENCODING);
 
@@ -266,7 +281,22 @@ public class LogfileMonitor {
 		File file = new File(m_filePath);
 
 		if (m_fr == null) {
-			m_fr = openFile(file, true);
+			// ログローテートまたは、初回読み込みでなく、firstHeadOpenフラグがtrueかつFileNotFoundが前回おきた場合は先頭から読む
+			if(!m_initFlag && m_firstHeadOpenStatus && m_firstHeadOpen){
+				m_log.info("run() : openFile=" + file + ", init=false");
+				m_log.debug("run() : m_initFlag=" + m_initFlag);
+				m_log.debug("run() : m_firstHeadOpenStatus=" + m_firstHeadOpenStatus);
+				m_log.debug("run() : m_firstHeadOpen=" + m_firstHeadOpen);
+				m_fr = openFile(file, false);
+			}
+			// 初回読み込みは必ず末尾から読む
+			// 2回目以降かつ、firstHeadOpenフラグがfalseの場合は末尾から読む
+			else{
+				m_log.info("run() : openFile=" + file + ", init=true");
+				m_fr = openFile(file, true);
+			}
+			
+			// オープンできないと終了
 			if (m_fr == null) {
 				return;
 			}
@@ -280,6 +310,7 @@ public class LogfileMonitor {
 		}
 
 		boolean readSuccessFlg = true; // 増加分読み込み成功フラグ 
+		m_logrotate = false;			// ローテートフラグ
 
 		try {
 			m_tmp_filesize = m_fr.length(); // 現在監視しているファイのサイズを取得・・・（１）
@@ -289,7 +320,6 @@ public class LogfileMonitor {
 				// ファイルサイズがm_unchangedStatsPeriod秒間以上変わらなかったら、ファイル切り替わりチェック
 				if ((++n_unchanged_stats * this.m_runInterval) >= this.m_unchangedStatsPeriod) {
 					m_log.debug("run() : " + m_filePath + " check log rotation");
-					boolean logrotate = false;
 
 					// ログローテートされているかチェックする
 					// 従来の判定ロジック：
@@ -297,7 +327,12 @@ public class LogfileMonitor {
 					// mv方式によりローテートされたと判断する。
 					if (m_fr.length() != file.length()) { // ・・・（２）
 						m_log.debug("run() : " + m_filePath + " file size not match");
-						logrotate = true;
+						m_logrotate = true;
+
+						m_log.debug("run() : m_logrotate set true .1");// rmしたとき、このルートでrotateしたと判断される！！
+						m_log.debug("run() : m_fr.length()=" + m_fr.length());
+						m_log.debug("run() : file.length()=" + file.length());
+						
 					} else if (m_tmp_filesize > 0 && m_firstPartDataCheckPeriod > 0 &&
 							(System.currentTimeMillis() - m_lastDataCheck) > m_firstPartDataCheckPeriod){
 						m_log.debug("run() : " + m_filePath + " check first part of file");
@@ -352,7 +387,8 @@ public class LogfileMonitor {
 									// ファイルの冒頭部分が異なれば別ファイルと判定
 									if (!Arrays.equals(refFirstPartOfFile, newFirstPartOfFile)) {
 										m_log.debug("run() : " + m_filePath + " log rotation detected");
-										logrotate = true;
+										m_logrotate = true;
+										m_log.debug("run() : m_logrotate set true .2");
 									}
 								}
 							}
@@ -378,7 +414,7 @@ public class LogfileMonitor {
 					}
 
 					// ログローテートされたと判定された場合
-					if(logrotate){
+					if(m_logrotate){
 						// 再度ファイルサイズの増加を確認する。
 						// ローテートされたと判定されたが、実は、ローテートされておらず、
 						//  （１）の時点では最終ログ読み込み時からログが出力されていないためサイズ（filesize）は同一であったが、
@@ -569,6 +605,12 @@ public class LogfileMonitor {
 				break;
 			}
 			for(MonitorStringValueInfo stringInfo : monitorInfo.getStringValueInfo()) {
+				if(m_log.isDebugEnabled()){
+					m_log.debug("patternMatch() line = " + line
+							+ ", monitorId = " + stringInfo.getMonitorId()
+							+ ", orderNo = " + stringInfo.getOrderNo()
+							+ ", pattern = " + stringInfo.getPattern());
+				}
 				if (!stringInfo.isValidFlg()) {
 					continue;
 				}
@@ -746,19 +788,26 @@ public class LogfileMonitor {
 			if (init) {
 				fr.seek(filesize);
 			}
+			
+			// 一度以上ファイルオープンに成功したら、フラグを無効化する
+			m_firstHeadOpenStatus = false;
 		} catch (FileNotFoundException e) {
 			m_log.info("openFile : " + e.getMessage());
 			if (m_initFlag) {
-				if (this.m_existenceFlg) {
-					// 最初にファイルをチェックする場合、監視管理へ通知
-					Object[] args = { this.m_filePath };
-					sendMessage(PriorityConstant.TYPE_INFO,
-							Messages.getString("agent"),
-							MESSAGE_ID_INFO,
-							Messages.getString("message.log.agent.2"),
-							Messages.getString("message.log.agent.1", args));
-				}
+				// 最初にファイルをチェックする場合、監視管理へ通知
+				Object[] args = { this.m_filePath };
+				sendMessage(PriorityConstant.TYPE_INFO,
+						Messages.getString("agent"),
+						MESSAGE_ID_INFO,
+						Messages.getString("message.log.agent.2"),
+						Messages.getString("message.log.agent.1", args));
 			}
+			
+			/*
+			// 一度ファイルが見えなくなったら、次回オープン時は先頭から(仕様変更となるため実施しない)
+			m_filesize = 0;
+			m_firstHeadOpenStatus = true;
+			*/
 		} catch (SecurityException e) {
 			m_log.info("openFile : " + e.getMessage());
 			if (m_initFlag) {
