@@ -17,6 +17,7 @@ AGENT_LIB=${HINEMOS_AGENT_HOME}/lib
 AGENT_CONF=${HINEMOS_AGENT_HOME}/conf
 TIMEOUT=3
 export PROG=`basename $0`
+export HINEMOS_AGENT_HOME=${HINEMOS_AGENT_HOME}
 
 # Logging
 function Logging() {
@@ -53,6 +54,20 @@ then
         fi
 fi
 
+# check java version
+if [ `${JAVA_HOME}/bin/java -version 2>&1 | grep '%%JAVA_CHECK_REGEX%%' | wc -l` -eq 0 ]
+then
+        Logging "This Java is not supported."
+        ${JAVA_HOME}/bin/java -version 2>&1
+        exit 1
+fi
+if [ `${JAVA_HOME}/bin/java -version 2>&1 | grep %%OPENJDK_CHECK_REGEX%% | wc -l` -eq 0 ]
+then
+        Logging "This Java is not supported."
+        ${JAVA_HOME}/bin/java -version 2>&1
+        exit 1
+fi
+
 ## tune JVM Heap Memory
 JAVA_OPTS="-Xms32m -Xmx32m"
 
